@@ -22,6 +22,7 @@ UM0034 = 0
 UM0017 = 1
 UM0090 = 2
 FS000x = 3
+FS0002 = 4
 
 
 #from PyQt5 import QtCore, QtGui, QtWidgets
@@ -45,6 +46,7 @@ class Ui_MainWindow(object):
         self.demoTypeComboBox.addItem("")
         self.demoTypeComboBox.addItem("")
         self.demoTypeComboBox.addItem("")
+        self.demoTypeComboBox.addItem("")
         self.horizontalLayout.addWidget(self.demoTypeComboBox)
         self.ComPortComboBox = QtWidgets.QComboBox(self.centralwidget)
         self.ComPortComboBox.setObjectName("ComPortComboBox")
@@ -100,11 +102,12 @@ class Ui_MainWindow(object):
 
     def retranslateUi(self, MainWindow):
         _translate = QtCore.QCoreApplication.translate
-        MainWindow.setWindowTitle(_translate("MainWindow", "Ultrasonic USB Demo           Version 1.6.7"))
+        MainWindow.setWindowTitle(_translate("MainWindow", "Ultrasonic USB Demo           Version 1.6.9"))
         self.demoTypeComboBox.setItemText(0, _translate("MainWindow", "UM0034-002 Proximity"))
         self.demoTypeComboBox.setItemText(1, _translate("MainWindow", "UM0017 Water Level"))
         self.demoTypeComboBox.setItemText(2, _translate("MainWindow", "UM0090 Proximity"))
         self.demoTypeComboBox.setItemText(3, _translate("MainWindow", "FS000x Smart Flow Meter"))
+        self.demoTypeComboBox.setItemText(4, _translate("MainWindow", "FS0002 Smart Flow Meter"))
         self.ComOpen.setText(_translate("MainWindow", "Open"))
         self.rescanButton.setText(_translate("MainWindow", "ReScan"))
 
@@ -152,7 +155,8 @@ class commVariables(object):
         self.runsecs = 0
         self.runmins = 0
         self.runhrs = 0
-        self.logfilename = ""
+#        self.logfilename = ""
+        self.togglemode = 0
 
 
 def isportopen():
@@ -368,6 +372,85 @@ class MainWindow(QMainWindow):
                 self.ui.parmbutton[node.index].setText('{0:d}'.format(node.value))
                 self.ui.parmbutton[node.index].show()
 
+#            self.ui.togglemodebutton = QtWidgets.QPushButton(self.ui.buttonFrame)
+#            self.ui.togglemodebutton.setGeometry(QtCore.QRect(260, 5, 100, 25))
+#            self.ui.togglemodebutton.setText('MToggle')
+#            self.ui.togglemodebutton.show()
+#            self.ui.togglemodebutton.pressed.connect(lambda n="call": self.ToggleModeButton(n))
+
+#            self.ui.resetaccbutton = QtWidgets.QPushButton(self.ui.buttonFrame)
+#            self.ui.resetaccbutton.setGeometry(QtCore.QRect(260, 35, 100, 25))
+#            self.ui.resetaccbutton.setText('ResetAcc')
+#            self.ui.resetaccbutton.show()
+#            self.ui.resetaccbutton.pressed.connect(lambda n="call": self.ResetAccButton(n))
+
+
+        elif(gcvars.demoType == FS0002):
+            fn = os.path.join(application_path, 'image/fs000x_banner.png')
+            self.ui.labelImage.setPixmap(QPixmap(fn))
+            fn = os.path.join(application_path, 'fs0002.xml')
+
+            with open(fn, 'rt') as f:
+                tree = ElementTree.parse(f)
+                root = tree.getroot()
+                for child in root.iter('parameter'):
+                    newparm = UltrasonicParameter()
+                    newparm.index = int(child.attrib['index'])
+                    newparm.readonly = int(child.attrib['readonly'])
+                    newparm.showhex = int(child.attrib['showhex'])
+                    newparm.value = int(child.attrib['initval'])
+                    newparm.name = child.text
+                    gcvars.UltrasonicParmList.append(newparm)
+                    print(newparm.index, child.text)
+
+            for item in gcvars.UltrasonicParmList:
+                print(item.index, item.readonly, item.showhex, item.value)
+
+
+            for node in gcvars.UltrasonicParmList:
+                xloc = 10 + (node.index // 20)*240
+                yloc = 5 + (node.index % 25)*30
+                self.ui.parmlabel[node.index] = QtWidgets.QLabel(self.ui.buttonFrame)
+                self.ui.parmlabel[node.index].setGeometry(QtCore.QRect(xloc, yloc, 120, 25))
+                self.ui.parmlabel[node.index].setObjectName("labelp")
+                self.ui.parmlabel[node.index].setText(node.name)
+                self.ui.parmlabel[node.index].show()
+                self.ui.parmbutton[node.index] = QtWidgets.QPushButton(self.ui.buttonFrame)
+                self.ui.parmbutton[node.index].setGeometry(QtCore.QRect(xloc+130, yloc, 100, 25))
+                self.ui.parmbutton[node.index].setText('{0:d}'.format(node.value))
+                self.ui.parmbutton[node.index].show()
+
+            self.ui.togglemodebutton = QtWidgets.QPushButton(self.ui.buttonFrame)
+            self.ui.togglemodebutton.setGeometry(QtCore.QRect(250, 5, 100, 25))
+            self.ui.togglemodebutton.setText('MToggle')
+            self.ui.togglemodebutton.show()
+            self.ui.togglemodebutton.pressed.connect(lambda n="call": self.ToggleModeButton(n))
+
+
+
+    def ToggleModeButton(self, s):
+        global gcvars
+        if(isportopen()):
+
+            print("Transmitting Toggle : "+format(gcvars.togglemode))
+            out_buffer = ""
+            if(gcvars.togglemode == 0):
+               temp_buffer = chr(0x10) + chr(0x5C) + chr(0x00) + chr(0x5C) +chr(0x16)
+               gcvars.togglemode = 1
+            else:
+               temp_buffer = chr(0x10) + chr(0x5C) + chr(0x01) + chr(0x5D) +chr(0x16)
+               gcvars.togglemode = 0
+
+            for i in range(0,96):
+                temp_buffer = chr(0xFE) + temp_buffer
+            self.ui.ReceiveMemo.appendPlainText('Sending : '+"".join(format(ord(x),'02x') for x in temp_buffer[96:]))
+            packet = bytearray()
+            for x in temp_buffer :
+                packet.append(ord(x))
+            gcvars.ser.write(packet)
+           
+            gcvars.ser.flushOutput()
+
 
     def ComOpenClick(self, s):
         global gcvars
@@ -389,11 +472,14 @@ class MainWindow(QMainWindow):
                 gcvars.ser.baudrate = 9600
                 gcvars.ser.parity = 'N'
             if(gcvars.demoType == UM0090):
-                gcvars.ser.baudrate = 9000
+                gcvars.ser.baudrate = 9600
                 gcvars.ser.parity = 'N'
             if(gcvars.demoType == FS000x):
                 gcvars.ser.baudrate = 115200
                 gcvars.ser.parity = 'E'
+            if(gcvars.demoType == FS0002):
+                gcvars.ser.baudrate = 9600
+                gcvars.ser.parity = 'E'
             
             gcvars.ser.open()
 
@@ -465,6 +551,30 @@ class MainWindow(QMainWindow):
            
             gcvars.ser.flushOutput()
 
+    def format_fs0002_outmessage(self, instr, strlen):
+        if isportopen() :
+#        if(gcvars.ser.is_open):
+            #        if(gcvars.ser.isOpen()):
+            print("Transmitting : ",instr)
+            out_csum = 0
+            out_buffer = ""
+            for from_index in range(0, strlen):
+                out_buffer = out_buffer + instr[from_index]
+                out_csum = out_csum + ord(instr[from_index])
+            if strlen == 1 : 
+                out_buffer = out_buffer + chr(0xFD)
+                out_csum = out_csum + 0xFD
+            temp_buffer = chr(0x10) + out_buffer + chr(out_csum & 0xFF) + chr(0x16)
+
+#            for i in range(0,96):
+#                temp_buffer = chr(0xFE) + temp_buffer
+            self.ui.ReceiveMemo.appendPlainText('Sending : '+"".join(format(ord(x),'02x') for x in temp_buffer))
+            packet = bytearray()
+            for x in temp_buffer :
+                packet.append(ord(x))
+            gcvars.ser.write(packet)
+           
+            gcvars.ser.flushOutput()
 
 
     def tick(self):
@@ -494,8 +604,9 @@ class MainWindow(QMainWindow):
             while(gcvars.ser.inWaiting() > 0):
                 char = gcvars.ser.read(1)
                 gcvars.receive_buffer[gcvars.receive_index] = ord(char)
-                print(ord(char))
-                print(char)
+                print(hex(ord(char))+" "+str(ord(char)))
+#                print(ord(char))
+#                print(char)
                 gcvars.receive_index += 1
                 gcvars.receive_index &= 0xFF
             # end while serial char to read
@@ -505,13 +616,18 @@ class MainWindow(QMainWindow):
             if(gcvars.wait_reply_timer == 0):
                 gcvars.wait_reply_timer = 40 # two seconds
                 # will eventually need to send commands other than an update request, but for now, that's all we have
-                if(gcvars.demoType == 3):
+                if(gcvars.demoType == FS000x):
                     self.format_fs000x_outmessage('[',1)
+                elif(gcvars.demoType == FS0002):
+                    if(gcvars.togglemode == 0):                    
+                        self.format_fs0002_outmessage('[',1)
                 else:
                     print("Transmitting : T")
                     gcvars.ser.write(ord('T'))
                     gcvars.ser.flushOutput()
 
+            if(gcvars.demoType == FS0002):
+                self.parseReceive_fs0002()
             if(gcvars.demoType == FS000x):
                 self.parseReceive_fs000x()
             if(gcvars.demoType == UM0090):
@@ -723,6 +839,171 @@ class MainWindow(QMainWindow):
                                     gcvars.UltrasonicParmList[0].skipnext = True
                                         
 
+    def parseReceive_fs0002(self):
+        starttokenfound = False
+        endtokenfound = False
+        check_index = 0
+        check_end_index = 0
+        look_buffer = [0] * 256
+        look_buffer_index = 0
+        input_checksum = 0
+        temp_substring = ""
+        getindex = 0
+        index = 0
+
+        if(gcvars.read_index != gcvars.receive_index):
+#                print('Read Index = {0}, Receive Index = {1}'.format(gcvars.read_index,gcvars.receive_index))
+            check_index = gcvars.read_index
+            starttokenfound = False
+
+
+
+            while check_index != gcvars.receive_index and not starttokenfound:
+                if gcvars.receive_buffer[check_index] != 0x42:
+                    # discard characters that aren't start of message
+                    check_index += 1
+                    check_index &= 0xFF
+                    gcvars.read_index += 1
+                    gcvars.read_index &= 0xFF
+                else:
+                    # Got a potential message to parse
+                    print('Got Start of Message')
+                    starttokenfound = True
+                    check_end_index = check_index
+                    endtokenfound = False
+                    while (check_end_index != gcvars.receive_index) and (not endtokenfound):
+                        look_buffer[look_buffer_index] = gcvars.receive_buffer[check_end_index]
+                        look_buffer_index += 1
+                        if look_buffer_index > 32:
+                            # overflow exit
+                            endtokenfound = True
+                            gcvars.read_index = gcvars.receive_index
+                            check_end_index = gcvars.receive_index
+                            print('Overflow, length is ',look_buffer_index)
+                        elif gcvars.receive_buffer[check_end_index] != 0x16 and look_buffer_index == 32:
+                            print('Bad End Char ',gcvars.receive_buffer[check_end_index],' length is ',look_buffer_index)
+                            print("".join(format(x,'02x') for x in look_buffer[0:look_buffer_index-1]))
+                            endtokenfound = True
+                            gcvars.read_index = gcvars.receive_index
+                            check_end_index = gcvars.receive_index
+                        elif look_buffer_index < 32:
+#                                print('Adding Char ',ord(gcvars.receive_buffer[check_end_index]),' length is ',len(look_buffer))
+                            check_end_index += 1
+                            check_end_index &= 0xFF
+                        else:
+                            print('Got end of message')
+                            
+                            gcvars.read_index = check_end_index + 1
+                            gcvars.read_index &= 0xFF
+                            endtokenfound = True
+                            if look_buffer_index > 4:
+                                print("".join(format(x,'02x') for x in look_buffer[:32]))
+                                checksum_valid = True
+                                try:
+                                    input_checksum = look_buffer[30]
+                                    print('Input Checksum is ',input_checksum,' length is ',look_buffer_index)
+                                except:
+                                    self.ui.ReceiveMemo.appendPlainText('ERROR - input checksum invalid format '+look_buffer)
+                                    checksum_valid = False
+                                print(input_checksum)
+                                read_checksum = 0
+                                for index in range(2, look_buffer_index-3):
+                                    read_checksum += look_buffer[index]
+                                read_checksum &= 0xFF
+                                print("readcsum1",read_checksum)
+
+                                read_checksum2 = 0
+                                for index in range(0, look_buffer_index-2):
+                                    read_checksum2 += look_buffer[index]
+                                read_checksum2 &= 0xFF
+                                print("readcsum2",read_checksum2)
+                                if(input_checksum == read_checksum):
+                                    print("csumorg")
+                                elif(input_checksum == read_checksum2):
+                                    print("csumnew")
+                                if(input_checksum == read_checksum)or(input_checksum==read_checksum2):
+                                    print("".join(format(x,'02x') for x in look_buffer[0:look_buffer_index]))
+                                    self.ui.ReceiveMemo.appendPlainText('Received : '+"".join(format(x,'02x') for x in look_buffer[0:look_buffer_index]))
+                                    gcvars.UltrasonicParmList[0].value = look_buffer[5]
+                                    self.ui.parmbutton[0].setText(str(look_buffer[5]))
+
+                                    # Device ID
+                                    gcvars.UltrasonicParmList[0].value = 0
+                                    for x in range(2,7):
+                                        gcvars.UltrasonicParmList[0].value *= 256
+                                        gcvars.UltrasonicParmList[0].value += look_buffer[x]
+                                    self.ui.parmbutton[0].setText(str(format(gcvars.UltrasonicParmList[0].value,"08x")))
+
+                                    # Version
+                                    gcvars.UltrasonicParmList[1].value = look_buffer[7]
+                                    self.ui.parmbutton[1].setText(str(look_buffer[7]))
+
+                                    # Status Byte
+                                    gcvars.UltrasonicParmList[2].value = look_buffer[29]
+                                    self.ui.parmbutton[2].setText(str(format(gcvars.UltrasonicParmList[2].value,"02x")))
+
+
+                                    # Acc. Flow
+                                    gcvars.UltrasonicParmList[3].value = 0
+                                    multiplier = 1
+                                    for x in range(9,15):
+                                        tempval = look_buffer[x]
+#                                        print('A'+format(tempval,'02x'))
+                                        bcdval = (tempval&0x0F) + (tempval >> 4)*10
+                                        gcvars.UltrasonicParmList[3].value += bcdval*multiplier
+                                        print(gcvars.UltrasonicParmList[3].value)
+                                        multiplier *= 100
+                                    self.ui.parmbutton[3].setText(str(gcvars.UltrasonicParmList[3].value/1000.0))
+
+                                    # Ins. Flow
+                                    gcvars.UltrasonicParmList[4].value = 0
+                                    multiplier = 1
+                                    for x in range(16,20):
+                                        tempval = look_buffer[x]
+#                                        print('I'+format(tempval,'02x'))
+                                        bcdval = (tempval&0x0F) + (tempval >> 4)*10
+                                        gcvars.UltrasonicParmList[4].value += bcdval*multiplier
+                                        print(gcvars.UltrasonicParmList[4].value)
+                                        multiplier *= 100
+                                    self.ui.parmbutton[4].setText(str(gcvars.UltrasonicParmList[4].value/100.0))
+#                                    with open(gcvars.logfilename, 'a') as out:
+#                                        out.write(str(datetime.now())+",")
+#                                        out.write(str(gcvars.UltrasonicParmList[4].value/100.0))
+#                                        out.write('\r');
+
+                                    # Acc. Time
+                                    gcvars.UltrasonicParmList[5].value = 0
+                                    multiplier = 1
+                                    for x in range(21,25):
+                                        tempval = look_buffer[x]
+#                                        print('H'+format(tempval,'02x'))
+                                        bcdval = (tempval&0x0F) + (tempval >> 4)*10
+                                        gcvars.UltrasonicParmList[5].value += bcdval*multiplier
+                                        print(gcvars.UltrasonicParmList[5].value)
+                                        multiplier *= 100
+                                    self.ui.parmbutton[5].setText(str(gcvars.UltrasonicParmList[5].value))
+
+                                    # Temperature
+                                    gcvars.UltrasonicParmList[6].value = 0
+                                    multiplier = 1
+                                    for x in range(26,29):
+                                        tempval = look_buffer[x]
+#                                        print('T'+format(tempval,'02x'))
+                                        bcdval = (tempval&0x0F) + (tempval >> 4)*10
+                                        gcvars.UltrasonicParmList[6].value += bcdval*multiplier
+                                        multiplier *= 100
+                                    self.ui.parmbutton[6].setText(str(gcvars.UltrasonicParmList[6].value/100.0))
+
+
+                                    # Checksum
+                                    gcvars.UltrasonicParmList[7].value = look_buffer[30]
+                                    self.ui.parmbutton[7].setText(str(format(gcvars.UltrasonicParmList[7].value,"02x")))
+
+                                    gcvars.wait_reply_timer = 5                                            
+                                        
+
+                                else:
+                                    self.ui.ReceiveMemo.appendPlainText("Checksum Failed "+"".join(format(x,'02x') for x in look_buffer[0:look_buffer_index])+" Expected {:X}".format(read_checksum))
 
 
 
@@ -743,7 +1024,7 @@ class MainWindow(QMainWindow):
             check_index = gcvars.read_index
             starttokenfound = False
             while check_index != gcvars.receive_index and not starttokenfound:
-                if gcvars.receive_buffer[check_index] != 0x68:
+                if gcvars.receive_buffer[check_index] != 0x42:
                     # discard characters that aren't start of message
                     check_index += 1
                     check_index &= 0xFF
@@ -751,47 +1032,47 @@ class MainWindow(QMainWindow):
                     gcvars.read_index &= 0xFF
                 else:
                     # Got a potential message to parse
-#                        print('Got Start of Message')
+                    print('Got Start of Message')
                     starttokenfound = True
                     check_end_index = check_index
                     endtokenfound = False
                     while (check_end_index != gcvars.receive_index) and (not endtokenfound):
                         look_buffer[look_buffer_index] = gcvars.receive_buffer[check_end_index]
                         look_buffer_index += 1
-                        if look_buffer_index > 49:
+                        if look_buffer_index > 32:
                             # overflow exit
                             endtokenfound = True
                             gcvars.read_index = gcvars.receive_index
                             check_end_index = gcvars.receive_index
                             print('Overflow, length is ',look_buffer_index)
-                        elif gcvars.receive_buffer[check_end_index] != 0x16 and look_buffer_index == 49:
+                        elif gcvars.receive_buffer[check_end_index] != 0x16 and look_buffer_index == 32:
                             print('Bad End Char ',gcvars.receive_buffer[check_end_index],' length is ',look_buffer_index)
-                            print("".join(format(x,'02x') for x in look_buffer[0,look_buffer_index]))
+                            print("".join(format(x,'02x') for x in look_buffer[0:look_buffer_index-1]))
                             endtokenfound = True
                             gcvars.read_index = gcvars.receive_index
                             check_end_index = gcvars.receive_index
-                        elif look_buffer_index < 49:
+                        elif look_buffer_index < 32:
 #                                print('Adding Char ',ord(gcvars.receive_buffer[check_end_index]),' length is ',len(look_buffer))
                             check_end_index += 1
                             check_end_index &= 0xFF
                         else:
-#                                print('Got end of message')
+                            print('Got end of message')
                             
                             gcvars.read_index = check_end_index + 1
                             gcvars.read_index &= 0xFF
                             endtokenfound = True
                             if look_buffer_index > 4:
-                                print("".join(format(x,'02x') for x in look_buffer))
+                                print("".join(format(x,'02x') for x in look_buffer[:32]))
                                 checksum_valid = True
                                 try:
-                                    input_checksum = look_buffer[47]
+                                    input_checksum = look_buffer[30]
                                     print('Input Checksum is ',input_checksum,' length is ',look_buffer_index)
                                 except:
                                     self.ui.ReceiveMemo.appendPlainText('ERROR - input checksum invalid format '+look_buffer)
                                     checksum_valid = False
                                 print(input_checksum)
                                 read_checksum = 0
-                                for index in range(4, look_buffer_index-2):
+                                for index in range(2, look_buffer_index-3):
                                     read_checksum += look_buffer[index]
                                 read_checksum &= 0xFF
                                 print(read_checksum)
@@ -800,85 +1081,74 @@ class MainWindow(QMainWindow):
                                     self.ui.ReceiveMemo.appendPlainText('Received : '+"".join(format(x,'02x') for x in look_buffer[0:look_buffer_index]))
                                     gcvars.UltrasonicParmList[0].value = look_buffer[5]
                                     self.ui.parmbutton[0].setText(str(look_buffer[5]))
-                                    gcvars.UltrasonicParmList[1].value = 0
-                                    for x in range(7,11):
-                                        gcvars.UltrasonicParmList[1].value *= 256
-                                        gcvars.UltrasonicParmList[1].value += look_buffer[x]
-                                    self.ui.parmbutton[1].setText(str(format(gcvars.UltrasonicParmList[1].value,"08x")))
 
-                                    gcvars.UltrasonicParmList[2].value = 0
-                                    for x in range(11,13):
-                                        gcvars.UltrasonicParmList[2].value *= 256
-                                        gcvars.UltrasonicParmList[2].value += look_buffer[x]
-                                    self.ui.parmbutton[2].setText(str(format(gcvars.UltrasonicParmList[2].value,"04x")))
+                                    # Device ID
+                                    gcvars.UltrasonicParmList[0].value = 0
+                                    for x in range(2,7):
+                                        gcvars.UltrasonicParmList[0].value *= 256
+                                        gcvars.UltrasonicParmList[0].value += look_buffer[x]
+                                    self.ui.parmbutton[0].setText(str(format(gcvars.UltrasonicParmList[0].value,"08x")))
 
-                                    gcvars.UltrasonicParmList[3].value = look_buffer[13]
-                                    self.ui.parmbutton[3].setText(str(gcvars.UltrasonicParmList[3].value))
-                                    
-                                    gcvars.UltrasonicParmList[4].value = look_buffer[14]
-                                    self.ui.parmbutton[4].setText(str(gcvars.UltrasonicParmList[4].value))
+                                    # Version
+                                    gcvars.UltrasonicParmList[1].value = look_buffer[7]
+                                    self.ui.parmbutton[1].setText(str(look_buffer[7]))
 
-                                    gcvars.UltrasonicParmList[5].value = look_buffer[15]
-                                    self.ui.parmbutton[5].setText(str(gcvars.UltrasonicParmList[5].value))
-
-                                    gcvars.UltrasonicParmList[6].value = look_buffer[16]
-                                    self.ui.parmbutton[6].setText(str(format(gcvars.UltrasonicParmList[6].value,"02x")))
-
-                                    gcvars.UltrasonicParmList[7].value = 0
-                                    for x in range(17,19):
-                                        gcvars.UltrasonicParmList[7].value *= 256
-                                        gcvars.UltrasonicParmList[7].value += look_buffer[x]
-                                    self.ui.parmbutton[7].setText(str(format(gcvars.UltrasonicParmList[7].value,"04x")))
+                                    # Status Byte
+                                    gcvars.UltrasonicParmList[2].value = look_buffer[29]
+                                    self.ui.parmbutton[2].setText(str(format(gcvars.UltrasonicParmList[2].value,"02x")))
 
 
-                                    gcvars.UltrasonicParmList[8].value = 0
+                                    # Acc. Flow
+                                    gcvars.UltrasonicParmList[3].value = 0
+                                    multiplier = 1
+                                    for x in range(9,15):
+                                        tempval = look_buffer[x]
+#                                        print('A'+format(tempval,'02x'))
+                                        bcdval = (tempval&0x0F) + (tempval >> 4)*10
+                                        gcvars.UltrasonicParmList[3].value += bcdval*multiplier
+                                        print(gcvars.UltrasonicParmList[3].value)
+                                        multiplier *= 100
+                                    self.ui.parmbutton[3].setText(str(gcvars.UltrasonicParmList[3].value/1000.0))
+
+                                    # Ins. Flow
+                                    gcvars.UltrasonicParmList[4].value = 0
+                                    multiplier = 1
+                                    for x in range(16,20):
+                                        tempval = look_buffer[x]
+#                                        print('I'+format(tempval,'02x'))
+                                        bcdval = (tempval&0x0F) + (tempval >> 4)*10
+                                        gcvars.UltrasonicParmList[4].value += bcdval*multiplier
+                                        print(gcvars.UltrasonicParmList[4].value)
+                                        multiplier *= 100
+                                    self.ui.parmbutton[4].setText(str(gcvars.UltrasonicParmList[4].value/100.0))
+
+                                    # Acc. Time
+                                    gcvars.UltrasonicParmList[5].value = 0
                                     multiplier = 1
                                     for x in range(21,25):
                                         tempval = look_buffer[x]
-                                        print('A'+format(tempval,'02x'))
+#                                        print('H'+format(tempval,'02x'))
                                         bcdval = (tempval&0x0F) + (tempval >> 4)*10
-                                        gcvars.UltrasonicParmList[8].value += bcdval*multiplier
-                                        print(gcvars.UltrasonicParmList[8].value)
+                                        gcvars.UltrasonicParmList[5].value += bcdval*multiplier
+                                        print(gcvars.UltrasonicParmList[5].value)
                                         multiplier *= 100
-                                    self.ui.parmbutton[8].setText(str(gcvars.UltrasonicParmList[8].value/100.0))
+                                    self.ui.parmbutton[5].setText(str(gcvars.UltrasonicParmList[5].value))
 
-                                    gcvars.UltrasonicParmList[9].value = 0
+                                    # Temperature
+                                    gcvars.UltrasonicParmList[6].value = 0
                                     multiplier = 1
-                                    for x in range(27,31):
+                                    for x in range(26,29):
                                         tempval = look_buffer[x]
-                                        print('B'+format(tempval,'02x'))
+#                                        print('T'+format(tempval,'02x'))
                                         bcdval = (tempval&0x0F) + (tempval >> 4)*10
-                                        gcvars.UltrasonicParmList[9].value += bcdval*multiplier
-                                        print(gcvars.UltrasonicParmList[8].value)
+                                        gcvars.UltrasonicParmList[6].value += bcdval*multiplier
                                         multiplier *= 100
-                                    self.ui.parmbutton[9].setText(str(gcvars.UltrasonicParmList[9].value/100.0))
+                                    self.ui.parmbutton[6].setText(str(gcvars.UltrasonicParmList[6].value/100.0))
 
-                                    gcvars.UltrasonicParmList[10].value = 0
-                                    multiplier = 1
-                                    for x in range(33,37):
-                                        tempval = look_buffer[x]
-#                                           print('C'+format(tempval,'02x'))
-                                        bcdval = (tempval&0x0F) + (tempval >> 4)*10
-                                        gcvars.UltrasonicParmList[10].value += bcdval*multiplier
-                                        multiplier *= 100
-                                    self.ui.parmbutton[10].setText(str(gcvars.UltrasonicParmList[10].value))
 
-                                    gcvars.UltrasonicParmList[11].value = 0
-                                    multiplier = 1
-                                    for x in range(39,41):
-                                        tempval = look_buffer[x]
-#                                           print('D'+format(tempval,'02x'))
-                                        bcdval = (tempval&0x0F) + (tempval >> 4)*10
-                                        gcvars.UltrasonicParmList[11].value += bcdval*multiplier
-                                        multiplier *= 100
-                                    self.ui.parmbutton[11].setText(str(gcvars.UltrasonicParmList[11].value/100.0))
-                                    
-                                    gcvars.UltrasonicParmList[12].value = look_buffer[46]*256
-                                    gcvars.UltrasonicParmList[12].value += look_buffer[45]
-                                    self.ui.parmbutton[12].setText(str(format(gcvars.UltrasonicParmList[12].value,"04x")))
-
-                                    gcvars.UltrasonicParmList[13].value = look_buffer[47]
-                                    self.ui.parmbutton[13].setText(str(format(gcvars.UltrasonicParmList[13].value,"02x")))
+                                    # Checksum
+                                    gcvars.UltrasonicParmList[7].value = look_buffer[30]
+                                    self.ui.parmbutton[7].setText(str(format(gcvars.UltrasonicParmList[7].value,"02x")))
 
                                     gcvars.wait_reply_timer = 5                                            
                                         
